// Fill out your copyright notice in the Description page of Project Settings.


#include "TargetWalker.h"

#include "AIController.h"
#include "Components/CapsuleComponent.h"

// Sets default values
ATargetWalker::ATargetWalker()
{
 	// Set this character to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

	// Find an appropriate SkeletalMesh that we want to use for our character. In this case, we can use a default engine
	// asset. We could also have this as a class variable declared as UPROPERTY, then we would have to assign the asset
	// to our component in the BeginPlay method.
	TObjectPtr<USkeletalMesh> SkelMesh = ConstructorHelpers::FObjectFinder<USkeletalMesh>(
		TEXT("/Script/Engine.SkeletalMesh'/Engine/Tutorial/SubEditors/TutorialAssets/Character/TutorialTPP.TutorialTPP'")).Object;
	GetMesh()->SetSkeletalMesh(SkelMesh);

	// Since the parent class already has everything else for a simple character, we don't need to add anything else.
	// However, the skeletal mesh we use has root at its feet while this character's collision capsule has root at its
	// center, so we will align those (and also fix rotation of the mesh).
	float HalfHeight = GetCapsuleComponent()->GetUnscaledCapsuleHalfHeight();
	GetMesh()->SetRelativeLocation(FVector(0, 0, -HalfHeight));

	// Fix rotation by rotating 90 degrees around the Z axis.
	GetMesh()->AddRelativeRotation(FRotator::MakeFromEuler(FVector(0, 0, -90)));
}

// Called when the game starts or when spawned
void ATargetWalker::BeginPlay()
{
	Super::BeginPlay();

	// Get our current controller and try casting it to the AAIController class.
	// For safety, we should check if this succeeds, then we can use the controller
	// to send our character to the target AActor.
	TObjectPtr<AAIController> AIController = dynamic_cast<AAIController*>(GetController());
	if (AIController != nullptr)
	{
		AIController->MoveToActor(Target);
	}
}

// Called every frame
void ATargetWalker::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
}

// Called to bind functionality to input
void ATargetWalker::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
	Super::SetupPlayerInputComponent(PlayerInputComponent);
}

